/*****************************************************************************
* RTData.c -	This is the main program module
\*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <rt.h>

#include "RTData.h"

// global variables
RTHANDLE			hRootProcess;
DWORD				dwKtickInUsecs;
RTHANDLE			hMBOX_Signal;								//add
INIT_STRUCT			gInit;

/*****************************************************************************
* FUNCTION:			main
*
* DESCRIPTION:
*  This is the main program module.
*  It creates global objects  and all threads.
*  The main thread then waits for notifications and acts accordingly
\*****************************************************************************/
int main(int argc, char* argv[])
{
	SYSINFO			sysinfo;
	EVENTINFO		eiEventInfo;
#ifdef _DEBUG
	fprintf(stderr, "RTData started\n");
#endif

	// obtain handle of root process (cannot fail)
	hRootProcess	= GetRtThreadHandles(ROOT_PROCESS);

	// initialize the structure for cleaning up
	memset(&gInit, 0, sizeof(gInit));
	gInit.state		= BEFORE_INIT;

	// get low level tick length in usecs
	if (!CopyRtSystemInfo(&sysinfo))
		Fail("Cannot copy system info");
	dwKtickInUsecs	= 10000 / sysinfo.KernelTickRatio;
	if (dwKtickInUsecs == 0)
		Fail("Invalid low level tick length");

	// adjust process max priority (ignore error)
	// TODO adjust the 2nd parameter to a value closer to zero if you want to allow more priorities
	SetRtProcessMaxPriority(NULL_RTHANDLE, 150);

	// obtain main thread's handle
	gInit.hMain		= GetRtThreadHandles(THIS_THREAD);
	gInit.state		= INIT_BUSY;

	// attempt to catalog the thread but ignore error
	Catalog(NULL_RTHANDLE, gInit.hMain, "TMain");

	// catalog the handle of this process in the root process
	if (!Catalog(hRootProcess, GetRtThreadHandles(THIS_PROCESS), "RTData"))
		Fail("Cannot catalog process name");
	gInit.bCataloged = TRUE;

	// create mailbox and semaphore threads
	hMBOX_Signal = CreateRtMailbox(DATA_MAILBOX | FIFO_QUEUING);	//add
	if (hMBOX_Signal == BAD_RTHANDLE)								//add
		Fail("Cannot create signaling data mailbox");				//add

	if (BAD_RTHANDLE == CreateRtThread(160, (LPPROC)ServerMY_MBOX_1, 4096, 0))
		Fail("Cannot create thread for MY_MBOX_1");
	if (BAD_RTHANDLE == CreateRtThread(160, (LPPROC)ServerMY_MBOX_2, 4096, 0))
		Fail("Cannot create thread for MY_MBOX_2");

	// create client threads
	if (BAD_RTHANDLE == CreateRtThread(180, (LPPROC)Client1, 4096, 0))
		Fail("Cannot create client thread Client1");

	// indicate that initialization has finished
	gInit.state		= INIT_DONE;
#ifdef _DEBUG
	fprintf(stderr, "RTData finished initialization\n");
#endif

	// wait for notifications
	while (RtNotifyEvent(RT_SYSTEM_NOTIFICATIONS | RT_EXIT_NOTIFICATIONS,
		WAIT_FOREVER, &eiEventInfo))
	{
		switch(eiEventInfo.dwNotifyType)
		{
		case TERMINATE:
			// TODO: this process should terminate
			// cleanup the environment
			Cleanup();  // does not return

		case NT_HOST_UP:
			// TODO: react to a Windows host that has come back
			break;

		case NT_BLUESCREEN:
			// TODO: react to a Windows blue screen
			break;

		case KERNEL_STOPPING:
			// TODO: react to the INtime kernel stopping
			break;

		case NT_HOST_HIBERNATE:
			// TODO: react to the Windows host going in hibernation
			break;

		case NT_HOST_STANDBY:
			// TODO: react to the Windows host going in standby mode
			break;

		case NT_HOST_SHUTDOWN_PENDING:
			// TODO: react to a Windows host that is about to shutdown
			break;
		}
	}
	Fail("Notify failed");
	return 0;
}
